// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:
#ifndef DUNE_TECTONIC_LINESEARCHSOLVER_HH
#define DUNE_TECTONIC_LINESEARCHSOLVER_HH

#include <dune/tnnmg/problem-classes/bisection.hh>

#include "../../utils/almostequal.hh"

/**
 * \brief A local solver for scalar quadratic obstacle problems with nonlinearity
 * using bisection
 */
class LineSearchSolver
{
public:
  template<class Vector, class Functional, class BitVector>
  void operator()(Vector& x, const Functional& f, const BitVector& ignore) const {
    x = 0;

    Dune::Solvers::Interval<double> D;
    D = f.subDifferential(0);
    if (D[1] > 0) // NOTE: Numerical instability can actually get us here
        return;

    if (almost_equal(f.domain()[0], f.domain()[1], 2)) {
        x = f.domain()[0];
        return;
    }

    if (not ignore) {
        int bisectionsteps = 0;
        const Bisection globalBisection;
        std::cout << "LineSearchSolver: starting bisection" << std::endl;
        x = globalBisection.minimize(f, 1.0, 0.0, bisectionsteps);
        std::cout << "LineSearchSolver: bisection terminated" << std::endl;
        // x = globalBisection.minimize(f, vNorm, 0.0, bisectionsteps) / vNorm;  // used rescaled v in f?
    }
  }
};

#endif

